const { spawn } = require('child_process')
const puppeteer = require('puppeteer')
const { exec } = require('../db/database');

function getHigherVulnerability(cves) {
  let severitiesArray = cves.map(cve => cve.cvssV3.severity)

  if (severitiesArray.includes('CRITICAL'))
    return 'CRITICAL'
  if (severitiesArray.includes('HIGH'))
    return 'HIGH'
  if (severitiesArray.includes('LOW'))
    return 'LOW'
  if (severitiesArray.includes('NONE'))
    return 'NONE'
  return 'N/A'
}

async function searchSploit(term) {
  const child = spawn('searchsploit', ['-j', '--id', `${term}`])

  let fullData = null
  child.stdout.on('data', (data) => {
    fullData += data.toString()
  })

  child.stderr.on('error', (stderr) => console.error('Standard Error: ', stderr))

  child.on('error', (error) => console.error(' Error: ', data))

  child.stdout.on('end', async () => {

    fullData = fullData.replaceAll('null', '');
    fullData = JSON.parse(fullData)?.RESULTS_EXPLOIT;

    let data = []
    for (let exploitIndex = 0; exploitIndex < fullData.length; exploitIndex++) {
      let cves = await scrappingDbExploit(fullData[exploitIndex]['EDB-ID']);
      cves = cves.cves;
      console.log(`Getting CVEs details of ${fullData[exploitIndex]['EDB-ID']}...`);
      //Pesquisar na BD Local
      let cves_data = await getCveDetailsFromMongo(cves)

      if (!cves_data.length) {
        //Pesquisar no Nist
        for (let cveIndex = 0; cveIndex < cves.length; cveIndex++) {
          console.log(`CVE: ${cves[cveIndex].cve_id}`);
          let cve_details = await scrappingNist(cves[cveIndex].link)
          cves_data = [...cves_data, { ...cves[cveIndex], cve_details }]
        }
      }
      data = [...data, { severity: getHigherVulnerability(cves_data), ed_id: fullData[exploitIndex]['EDB-ID'], title: fullData[exploitIndex].Title, cves_data }]

      if (exploitIndex > 1) continue;
    }

    console.log(data);

  })

  child.on('exit', (code, signal) => {
    if (code) console.log(`Process exit with code ${code}`);
    if (signal) console.log(`Process killed with signal ${signal}`);
  })
}

async function scrappingDbExploit(exploitDbID) {
  // exploitDbID = '50512'
  console.log(`Searching CVE for ED-ID: ${exploitDbID}...`);
  try {
    const browser = await puppeteer.launch({ headless: true });
    const page = await browser.newPage();

    await page.goto(`https://www.exploit-db.com/exploits/${exploitDbID}`);

    page.setDefaultNavigationTimeout(60000)

    const cveSelector = 'body > div.wrapper > div.main-panel > div.content > div > div > div:nth-child(1) > div > div.ml-2.mr-2 > div:nth-child(1) > div:nth-child(1) > div > div.card-body > div > div > div > div:nth-child(2) > h6';

    await page.waitForSelector(cveSelector);

    const cves = await page.$$eval(cveSelector + ' > a', (cves) => cves.map((cve) => ({

      link: cve.getAttribute('href'),

      cve_id: cve.innerText
    }))).catch(err => console.log('not founded'))


    const exploitTypeSelector = 'body > div.wrapper > div.main-panel > div.content > div > div > div:nth-child(1) > div > div.ml-2.mr-2 > div:nth-child(1) > div:nth-child(2) > div > div.card-body > div > div > div > div:nth-child(2) > h6 > a';

    const exploitType = await page.$$eval(exploitTypeSelector, (cves) => cves[0].innerText)

    const exploitPlatformSelector = 'body > div.wrapper > div.main-panel > div.content > div > div > div:nth-child(1) > div > div.ml-2.mr-2 > div:nth-child(1) > div:nth-child(3) > div > div.card-body > div > div > div > div:nth-child(1) > h6 > a';

    const exploitPlatform = await page.$$eval(exploitPlatformSelector, (cves) => cves[0].innerText)

    await browser.close()
    return { cves, type: exploitType, platform: exploitPlatform }
  } catch (error) {
    return error
  }
}

async function scrappingNist(url) {
  console.log(`Searching CVE details in: ${url}...`);
  try {
    const browser = await puppeteer.launch({ headless: true });
    const page = await browser.newPage();

    await page.goto(`${url}`);

    page.setDefaultNavigationTimeout(60000)

    const cveSelector = '#vulnDetailTableView';

    await page.waitForSelector(cveSelector);

    const data = await page.$eval(cveSelector, (element) => ({

      description: element.querySelector('p[data-testid=vuln-description]').innerText,

      baseScore: element.querySelectorAll('.severityDetail > a')[0].innerText

    }))

    await browser.close()
    return data
  } catch (error) {
    return error
  }
}

async function getCveDetailsFromMongo(cves) {
  console.log(`Get CVEs details...`);
  try {
    const conditions = cves.map(cve => `CVE-${cve.cve_id}`)
    const result = await exec(async (conn) => {
      return await conn
        .db(process.env.MONGODB_NAME)
        .collection("cves")
        .aggregate([
          {
            $match: {
              cve_id: {
                $in: conditions
              }
            }
          },
          { $project: { _id: 0 } }
        ]).toArray();
    });
    return result
  } catch (error) {
    return error
  }
}


module.exports = {
  searchSploit
}
